--[[
Author: Starinnia
RealID Toons - Add character information to the BNet alerts your RealID friends generate
$Date: 2010-10-19 04:25:53 +0000 (Tue, 19 Oct 2010) $
$Revision: 83 $
Project Version: r83
contact: codemaster2010 AT gmail DOT com

Copyright (c) 2010 Michael J. Murray aka Lyte of Lothar(US)
All rights reserved unless otherwise explicitly stated.
]]

local format = string.format

--save for the future
local myrealm = GetRealmName()

--create a reverse lookup from localized classes to english names
--we need the english names to get the class colors
local reverseClassLookup = {}
for k, v in pairs(LOCALIZED_CLASS_NAMES_FEMALE) do
	reverseClassLookup[v] = k
end
for k, v in pairs(LOCALIZED_CLASS_NAMES_MALE) do
	reverseClassLookup[v] = k
end

--pass in the toonID of the friend
--from this we can obtain the character name, realm, class, and faction
local function constructToonName(toonID, presenceID, isChat)
	local _, toon, _, realm, faction, _, class = BNGetToonInfo(toonID)
	local note = select(12, BNGetFriendInfoByID(presenceID))
	local canCoop = CanCooperateWithToon(toonID)
	
	if RID_TOONS_COLOR then
		--pass the localized class into the lookup table
		local eClass = reverseClassLookup[class]
		local r, g, b
		if _G.CUSTOM_CLASS_COLORS and _G.CUSTOM_CLASS_COLORS[eClass] then
			r, g, b = CUSTOM_CLASS_COLORS[eClass].r*255, CUSTOM_CLASS_COLORS[eClass].g*255, CUSTOM_CLASS_COLORS[eClass].b*255
		elseif RAID_CLASS_COLORS[eClass] then
			r, g, b = RAID_CLASS_COLORS[eClass].r*255, RAID_CLASS_COLORS[eClass].g*255, RAID_CLASS_COLORS[eClass].b*255
		else
			--sometimes BNGetToonInfo does not return class info
			--when this occurs default to a gray color
			r, g, b = 0.63*255, 0.63*255, 0.63*255
		end
		
		if isChat and canCoop then
			toon = format("|Hplayer:%s|h|cff%02x%02x%02x%s|r|h", toon, r, g, b, toon)
		else
			toon = format("|cff%02x%02x%02x%s|r", r, g, b, toon)
		end
	end
	
	--make sure realm has a value, sometimes BNGetToonInfo returns an empty string
	if RID_TOONS_REALM and realm ~= myrealm and realm ~= "" then
		toon = toon.." - "..realm
	end
	
	if RID_TOONS_FACTION then
		if faction and (not canCoop) then
			if faction == 0 then
				toon = format("|TInterface\\PVPFrame\\PVP-Currency-Horde:14|t%s", toon)
			elseif faction == 1 then
				toon = format("|TInterface\\PVPFrame\\PVP-Currency-Alliance:16|t%s", toon)
			end
		end
	end
	
	if RID_TOONS_SHOWNOTE then
		if note and note ~= "" then
			toon = format("%s [%s]", toon, note)
		end
	end
	
	return toon
end


local function filter(self, event, msg, author, ...)
	--the only toast we care about is FRIEND_ONLINE
	if msg == "FRIEND_ONLINE" then
		return true, msg, author, ...
	else
		return false, msg, author, ...
	end	
end
ChatFrame_AddMessageEventFilter("CHAT_MSG_BN_INLINE_TOAST_ALERT", filter)

local info = ChatTypeInfo["BN_INLINE_TOAST_ALERT"]
local savedChatFrame
local frame = CreateFrame("FRAME")
frame:RegisterEvent("BN_FRIEND_ACCOUNT_ONLINE")
frame:SetScript("OnEvent", function(self, event, presenceID)
	local _, givenName, surname, toon, toonID, client = BNGetFriendInfoByID(presenceID)
	local author = format(BATTLENET_NAME_FORMAT, givenName, surname)
	
	--determine which chat frame the alerts should go to
	if not savedChatFrame then
		for _, frame in ipairs(CHAT_FRAMES) do
			for _, msg in ipairs(_G[frame].messageTypeList) do
				if msg == "BN_INLINE_TOAST_ALERT" then
					savedChatFrame = _G[frame]
				end
			end
		end
	end
	if not savedChatFrame then savedChatFrame = DEFAULT_CHAT_FRAME end
	
	if client == BNET_CLIENT_WOW then
		local playerlink = format("|HBNplayer:%s:%s:%s:%s:%s|h[%s]|h", author, presenceID, 0, Chat_GetChatCategory("BN_INLINE_TOAST_ALERT"), 0, author)
		local prefix = format(BN_INLINE_TOAST_FRIEND_ONLINE, playerlink)
		--add a message to the passed in chat frame that looks like the regular toast
		savedChatFrame:AddMessage(format("%s (%s)", prefix, constructToonName(toonID, presenceID, true)), info.r, info.g, info.b, info.id)
	elseif client == BNET_CLIENT_SC2 then
		--construct the playerlink and message prefix seperately for easier debugging
		local playerlink = format("|HBNplayer:%s:%s:%s:%s:%s|h[%s]|h", author, presenceID, 0, Chat_GetChatCategory("BN_INLINE_TOAST_ALERT"), 0, author)
		local prefix = format(BN_INLINE_TOAST_FRIEND_ONLINE, playerlink)
		savedChatFrame:AddMessage(format("%s (|TInterface\\FriendsFrame\\Battlenet-Sc2icon:17|t %s)", prefix, toon), info.r, info.g, info.b, info.id)
	end
end)

hooksecurefunc("BNToastFrame_Show", function()
	BNToastFrame:SetWidth(250) -- Need to reset to original value for popups we are not modifying
	BNToastFrameGlowFrame.glow:SetWidth(252)
	if BNToastFrame.toastType ~= 1 then return end
	
	local _, _, _, toon, toonID, client = BNGetFriendInfoByID(BNToastFrame.toastData)
	
	if client == BNET_CLIENT_WOW then
		local originalText = BNToastFrameTopLine:GetText()
		local toonName = constructToonName(toonID, BNToastFrame.toastData, false)
		BNToastFrameTopLine:SetWidth(0)
		BNToastFrameTopLine:SetFormattedText("%s (%s)", originalText, toonName)
		--size the popup to match the text
		if (BNToastFrameTopLine:GetStringWidth() + 15) > (BN_TOAST_MAX_LINE_WIDTH - 10) then
			BNToastFrameTopLine:SetWidth(BNToastFrameTopLine:GetStringWidth() + 30)
			BNToastFrame:SetWidth(45+BNToastFrameTopLine:GetWidth())
			--make the animation glow effect fit the resized Toast popup
			BNToastFrameGlowFrame.glow:SetWidth(BNToastFrame:GetWidth()+2)
		end
	elseif client == BNET_CLIENT_SC2 then
		local originalText = BNToastFrameTopLine:GetText()
		BNToastFrameTopLine:SetWidth(0)
		BNToastFrameTopLine:SetFormattedText("%s - |TInterface\\FriendsFrame\\Battlenet-Sc2icon:17|t %s", originalText, toon)
		--size the popup to match the text
		if (BNToastFrameTopLine:GetStringWidth() + 15) > (BN_TOAST_MAX_LINE_WIDTH - 10) then
			BNToastFrameTopLine:SetWidth(BNToastFrameTopLine:GetStringWidth() + 30)
			BNToastFrame:SetWidth(45+BNToastFrameTopLine:GetWidth())
			--make the animation glow effect fit the resized Toast popup
			BNToastFrameGlowFrame.glow:SetWidth(BNToastFrame:GetWidth()+2)
		end
	end
end)

--create a custom popup menu for player right clicks on the
--custom links from the logon alerts
--using default was broken by the new privacy features
--and hooking led to taint issues
local popupMenuFrame
do
	local function closeAll() CloseDropDownMenus() end
	
	local function doWhisper(dropdownbutton, name, chatFrame, checked)
		ChatFrame_SendTell(name, chatFrame)
	end
	
	local function doConversation(dropdownbutton, presenceID, arg2, checked)
		BNConversationInvite_NewConversation(presenceID)
	end
	
	local function doInvite(dropdownbutton, name, arg2, checked)
		InviteUnit(name)
	end
	
	local function UncheckHack(dropdownbutton)
		_G[dropdownbutton:GetName().."Check"]:Hide()
	end
	
	local function doBNetReport(dropdownbutton, presenceID, kind, checked)
		BNet_InitiateReport(presenceID, kind)
	end
	
	popupMenuFrame = CreateFrame("FRAME", "RIDToons_PopupMenu")
	popupMenuFrame.displayMode = "MENU"
	local pMenuInfo = {}
	popupMenuFrame.initialize = function(self, level)
		if not level then return end
		wipe(pMenuInfo)
		
		if level == 1 then
			--title
			pMenuInfo.isTitle = 1
			pMenuInfo.text = self.author
			pMenuInfo.notCheckable = 1
			UIDropDownMenu_AddButton(pMenuInfo, level)
			
			pMenuInfo.disabled = nil
			pMenuInfo.isTitle = nil
			
			--whisper
			pMenuInfo.text = WHISPER
			pMenuInfo.func = doWhisper
			pMenuInfo.arg1 = self.author
			pMenuInfo.arg2 = self.chatFrame
			UIDropDownMenu_AddButton(pMenuInfo, level)
			
			--create conversation
			pMenuInfo.text = CREATE_CONVERSATION_WITH
			pMenuInfo.func = doConversation
			pMenuInfo.arg1 = self.presenceID
			pMenuInfo.arg2 = nil
			UIDropDownMenu_AddButton(pMenuInfo, level)
			
			--Invite
			pMenuInfo.text = INVITE
			pMenuInfo.func = doInvite
			pMenuInfo.arg1 = self.character
			pMenuInfo.arg2 = nil
			pMenuInfo.disabled = (not CanCooperateWithToon(self.characterID)) and 1
			UIDropDownMenu_AddButton(pMenuInfo, level)
			
			pMenuInfo.disabled = nil
			--pMenuInfo.notCheckable = nil
			pMenuInfo.keepShownOnClick = 1
			
			--Report
			pMenuInfo.text = BNET_REPORT
			pMenuInfo.hasArrow = 1
			pMenuInfo.value = "reportMenu"
			pMenuInfo.func = UncheckHack
			UIDropDownMenu_AddButton(pMenuInfo, level)
			
			pMenuInfo.value = nil
			pMenuInfo.hasArrow = nil
			
			--close the menu
			pMenuInfo.text = CANCEL
			pMenuInfo.func = closeAll
			pMenuInfo.checked = nil
			pMenuInfo.keepShownOnClick = nil
			pMenuInfo.notCheckable = 1
			UIDropDownMenu_AddButton(pMenuInfo, level)
		elseif level == 2 then
			if UIDROPDOWNMENU_MENU_VALUE == "reportMenu" then
				pMenuInfo.notCheckable = nil
				
				--report spam
				pMenuInfo.text = BNET_REPORT_SPAM
				pMenuInfo.func = doBNetReport
				pMenuInfo.arg1 = self.presenceID
				pMenuInfo.arg2 = "SPAM"
				UIDropDownMenu_AddButton(pMenuInfo, level)
				
				--report abuse
				pMenuInfo.text = BNET_REPORT_ABUSE
				pMenuInfo.arg2 = "ABUSE"
				UIDropDownMenu_AddButton(pMenuInfo, level)
				
				--report name
				pMenuInfo.text = BNET_REPORT_NAME
				pMenuInfo.arg2 = "NAME"
				UIDropDownMenu_AddButton(pMenuInfo, level)
			end
		end
	end
end
--we have a custom link type to avoid tainting the dropdown
--this allows the addon to circumvent the issues with the FriendsFrame_ShowBNDropdown
--call that is in ItemRef.lua of the default UI
--|HRIDT:presenceID:name|h[name]|h
local oldSetItemRef = SetItemRef
function SetItemRef(link, text, button, chatFrame)
	local linktype, presenceID, name = strsplit(":", link)
	if linktype == "RIDT" then
		if button == "RightButton" then
			if not BNIsSelf(presenceID) then
				local _, _, _, toon, toonID = BNGetFriendInfoByID(presenceID)
				popupMenuFrame.author = name
				popupMenuFrame.chatFrame = chatFrame
				popupMenuFrame.presenceID = presenceID
				popupMenuFrame.character = toon
				popupMenuFrame.characterID = toonID
				ToggleDropDownMenu(1, nil, RIDToons_PopupMenu, "cursor");
			end
		elseif button == "LeftButton" then
			ChatFrame_SendTell(name, chatFrame)
		end
	else
		return oldSetItemRef(link, text, button, chatFrame)
	end
end
